package run.evan.modemtool;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.telephony.CellIdentityNr;
import android.telephony.CellInfo;
import android.telephony.CellInfoNr;
import android.telephony.CellSignalStrengthNr;
import android.telephony.SubscriptionManager;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.qualcomm.qcrilhook.QcRilHookCallback;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.List;

public class MainActivity extends Activity {


    private static final String TAG = "ModemTool";

    TelephonyManager telephonyManager;

    Button btnApply;

    Button btnRead;

    Button btnWrite;

    Button btnDelete;

    Button btnRefresh;

    EditText editText;

    EditText editPci;

    EditText editArfcn;

    EditText editBand;


    Spinner spinner;


    private MiRilHook miRilHook = null;

    private boolean miRilHookReady = false;


    TelephonyManager.CellInfoCallback cellInfoCallback;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        editText = findViewById(R.id.editTextTextMultiLine);
        initEnv();

        spinner = findViewById(R.id.spinner);
        spinner.setSelection(1);


        editPci = findViewById(R.id.edit_pci);
        editArfcn = findViewById(R.id.edit_arfcn);
        editBand = findViewById(R.id.edit_band);

        btnRead = findViewById(R.id.btn_read);
        btnRead.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                readEFSToUI();
            }
        });

        btnWrite = findViewById(R.id.btn_write);
        btnWrite.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                writeUIToEFS();
            }
        });


        btnDelete = findViewById(R.id.btn_delete);
        btnDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                delete();
            }
        });

        btnRefresh = findViewById(R.id.btn_refresh);
        btnRefresh.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                query5GInfo();
            }
        });

        btnApply = findViewById(R.id.btn_apply);
        btnApply.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                refreshMbn();
            }
        });

        cellInfoCallback = new TelephonyManager.CellInfoCallback() {
            @Override
            public void onCellInfo(List<CellInfo> allCellInfo) {
                for (CellInfo cellInfo : allCellInfo) {
                    if (cellInfo instanceof CellInfoNr) {
                        if(cellInfo.isRegistered()){
                            CellInfoNr nrInfo = (CellInfoNr) cellInfo;
                            CellIdentityNr cellIdentity = (CellIdentityNr) nrInfo.getCellIdentity();
                            CellSignalStrengthNr cellSignalStrength =(CellSignalStrengthNr) nrInfo.getCellSignalStrength();

                            editArfcn.setText(String.valueOf(cellIdentity.getNrarfcn()));
                            editPci.setText(String.valueOf(cellIdentity.getPci()));
                            editBand.setText(String.valueOf(cellIdentity.getBands()[0]));
                            editText.setText("RSRP="+cellSignalStrength.getSsRsrp()+",RSRQ="+cellSignalStrength.getSsRsrq()+",SINR="+cellSignalStrength.getSsSinr());
                            return;
                        }
                    }
                }
            }
        };
    }

    private void initEnv() {
        try {
            telephonyManager = (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
            miRilHook = new MiRilHook(getApplicationContext(), new QcRilHookCallback() {
                @Override
                public void onQcRilHookDisconnected() {
                    Log.i(TAG, "onMiRilHookDisconnected");
                    miRilHookReady = false;
                }

                @Override
                public void onQcRilHookReady() {
                    Log.i(TAG, "onQcRilHookReady");
                    miRilHookReady = true;
                }
            });



        } catch (Exception e) {
            editText.setText(e.toString());
        }

    }



    private void refreshMbn() {
        try {
            if (miRilHookReady) {
                int slotIndex = SubscriptionManager.getSlotIndex(getSub());
                miRilHook.qcRilRefreshMbn(slotIndex);
            } else {
                Toast.makeText(getApplicationContext(), "mQcRilHookReady = false", Toast.LENGTH_LONG).show();
            }
        } catch (Exception e) {

        }
    }


    private void readEFSToUI() {
        try {
            int slotIndex = SubscriptionManager.getSlotIndex(getSub());
            ByteBuffer buf = miRilHook.onHookBigEfsReadSync(slotIndex, "/nv/item_files/modem/nr5g/RRC/pci_lock_info");
            if (null == buf) {
                editText.setText("buf is null");
            } else {
                int ret = buf.getInt();
                int len = buf.getInt();
                byte[] bytes = null;
                if (len > 0 && ret == 0) {
                    if (len != 12) {
                        editText.setText("pci_lock_info len != 12");
                        return;
                    }
                    bytes = new byte[len];
                    buf.get(bytes);
                } else if (ret != 0) {
                    editPci.setText("");
                    editArfcn.setText("");
                    editBand.setText("");
                    return;
                }

                ByteBuffer buffer = ByteBuffer.wrap(bytes);
                buffer.order(ByteOrder.LITTLE_ENDIAN);
                editPci.setText(String.valueOf(buffer.getInt()));
                editArfcn.setText(String.valueOf(buffer.getInt()));
                short scs = buffer.getShort();
                spinner.setSelection(scs);
                editBand.setText(String.valueOf(buffer.getShort() + 1));
            }
        } catch (Exception e) {
            editText.setText(e.toString());
        }
    }

    private void writeUIToEFS() {
        try {
            int pci = Integer.parseInt(editPci.getText().toString());
            int arfcn = Integer.parseInt(editArfcn.getText().toString());
            short band = (short) Integer.parseInt(editBand.getText().toString());
            band--;
            short scs = (short) spinner.getSelectedItemPosition();
            ByteBuffer buffer = ByteBuffer.allocate(12);
            buffer.order(ByteOrder.LITTLE_ENDIAN);
            buffer.putInt(pci);
            buffer.putInt(arfcn);
            buffer.putShort(scs);
            buffer.putShort(band);
            int slotIndex = SubscriptionManager.getSlotIndex(getSub());
            miRilHook.onHookEfsWriteSync(slotIndex, "/nv/item_files/modem/nr5g/RRC/pci_lock_info", buffer.array());
        } catch (Exception e) {
            editText.setText(e.toString());
        }
    }

    private void delete() {
        try {
            miRilHook.onHookEfsOptSync(0, "/nv/item_files/modem/nr5g/RRC/pci_lock_info", 6);
            miRilHook.onHookEfsOptSync(1, "/nv/item_files/modem/nr5g/RRC/pci_lock_info", 6);
        } catch (Exception e) {
            editText.setText(e.toString());
        }
    }


    private void query5GInfo() {
        telephonyManager.createForSubscriptionId(getSub()).requestCellInfoUpdate(getMainExecutor(),cellInfoCallback );
    }


    private String byteArrayToString(byte[] byteArray) {
        StringBuilder sb = new StringBuilder();
        for (byte b : byteArray) {
            sb.append(b).append(" ");
        }
        return sb.toString();
    }

    private int getSub() {
        int subscriberId = 0;
        subscriberId = SubscriptionManager.getDefaultDataSubscriptionId();
        return subscriberId;
    }





}